@@ -20,7 +20,7 @@ sub import {
     else {
         $metaclass = shift;
         ($metaclass->isa('Class::MOP::Class'))
-            || confess 'The metaclass must be derived from Class::MOP::Class';        
+            || confess "The metaclass ($metaclass) must be derived from Class::MOP::Class";
     }
     my %options = @_;
     my $package = caller();
@@ -79,10 +79,12 @@ This is a pragma to make it easier to use a specific metaclass
 and a set of custom attribute and method metaclasses. It also 
 installs a C<meta> method to your class as well. 
 
-=head1 AUTHOR
+=head1 AUTHORS
 
 Stevan Little E<lt>stevan@iinteractive.comE<gt>
 
+Yuval Kogman E<lt>nothingmuch@woobling.comE<gt>
+
 =head1 COPYRIGHT AND LICENSE
 
 Copyright 2006 by Infinity Interactive, Inc.
@@ -92,4 +94,4 @@ L<http://www.iinteractive.com>
 This library is free software; you can redistribute it and/or modify
 it under the same terms as Perl itself. 
 
-=cut
\ No newline at end of file
+=cut
